@@ -5,6 +5,8 @@
 // <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
+//
+// SPDX-License-Identifier: MIT/Apache-2.0
 
 use gst::glib;
 use gst::prelude::*;
@@ -15,9 +17,7 @@ use gst_video::subclass::prelude::*;
 
 use once_cell::sync::Lazy;
 
-use std::convert::TryInto;
 use std::i32;
-use std::str::FromStr;
 use std::sync::Mutex;
 
 #[derive(Default)]
@@ -80,7 +80,7 @@ impl Dav1dDec {
                 None => format_desc.into(),
             }
         };
-        gst_video::VideoFormat::from_str(&f).unwrap_or_else(|_| {
+        f.parse::<gst_video::VideoFormat>().unwrap_or_else(|_| {
             gst_warning!(CAT, "Unsupported dav1d format: {}", f);
             gst_video::VideoFormat::Unknown
         })
@@ -349,6 +349,8 @@ impl ObjectSubclass for Dav1dDec {
 
 impl ObjectImpl for Dav1dDec {}
 
+impl GstObjectImpl for Dav1dDec {}
+
 impl ElementImpl for Dav1dDec {
     fn metadata() -> Option<&'static gst::subclass::ElementMetadata> {
         static ELEMENT_METADATA: Lazy<gst::subclass::ElementMetadata> = Lazy::new(|| {
@@ -365,7 +367,7 @@ impl ElementImpl for Dav1dDec {
 
     fn pad_templates() -> &'static [gst::PadTemplate] {
         static PAD_TEMPLATES: Lazy<Vec<gst::PadTemplate>> = Lazy::new(|| {
-            let sink_caps = gst::Caps::new_simple("video/x-av1", &[]);
+            let sink_caps = gst::Caps::builder("video/x-av1").build();
             let sink_pad_template = gst::PadTemplate::new(
                 "sink",
                 gst::PadDirection::Sink,
@@ -374,21 +376,18 @@ impl ElementImpl for Dav1dDec {
             )
             .unwrap();
 
-            let src_caps = gst::Caps::new_simple(
-                "video/x-raw",
-                &[
-                    ("format", &gst::List::from_owned(video_output_formats())),
-                    ("width", &gst::IntRange::<i32>::new(1, i32::MAX)),
-                    ("height", &gst::IntRange::<i32>::new(1, i32::MAX)),
-                    (
-                        "framerate",
-                        &gst::FractionRange::new(
-                            gst::Fraction::new(0, 1),
-                            gst::Fraction::new(i32::MAX, 1),
-                        ),
+            let src_caps = gst::Caps::builder("video/x-raw")
+                .field("format", gst::List::from(video_output_formats()))
+                .field("width", gst::IntRange::new(1, i32::MAX))
+                .field("height", gst::IntRange::new(1, i32::MAX))
+                .field(
+                    "framerate",
+                    gst::FractionRange::new(
+                        gst::Fraction::new(0, 1),
+                        gst::Fraction::new(i32::MAX, 1),
                     ),
-                ],
-            );
+                )
+                .build();
             let src_pad_template = gst::PadTemplate::new(
                 "src",
                 gst::PadDirection::Src,
@@ -464,23 +463,11 @@ impl VideoDecoderImpl for Dav1dDec {
     fn decide_allocation(
         &self,
         element: &Self::Type,
-        query: &mut gst::QueryRef,
-    ) -> Result<(), gst::ErrorMessage> {
-        if let gst::query::QueryView::Allocation(allocation) = query.view() {
-            if allocation
-                .find_allocation_meta::<gst_video::VideoMeta>()
-                .is_some()
-            {
-                let pools = allocation.allocation_pools();
-                if let Some((Some(ref pool), _, _, _)) = pools.first() {
-                    let mut config = pool.config();
-                    config.add_option(&gst_video::BUFFER_POOL_OPTION_VIDEO_META);
-                    pool.set_config(config)
-                        .map_err(|e| gst::error_msg!(gst::CoreError::Negotiation, [&e.message]))?;
-                    self.negotiation_infos.lock().unwrap().video_meta_supported = true;
-                }
-            }
-        }
+        query: gst::query::Allocation<&mut gst::QueryRef>,
+    ) -> Result<(), gst::LoggableError> {
+        self.negotiation_infos.lock().unwrap().video_meta_supported = query
+            .find_allocation_meta::<gst_video::VideoMeta>()
+            .is_some();
 
         self.parent_decide_allocation(element, query)
     }
